/* eslint-disable @typescript-eslint/camelcase */
import React, { useState, useEffect } from "react";
import { connect } from "react-redux";
import { RouteComponentProps } from "react-router-dom";
import { Translate, translate } from "react-jhipster";
// import { IRootState } from "app/shared/reducers";
// import { getEntities as getChartViews } from "app/entities/chart-view/chart-view.reducer";
import { getEntity, updateEntity, createEntity, reset } from "./chart.reducer";
import Button from "@material-ui/core/Button";
import TextField from "@material-ui/core/TextField";
import {
  Grid,
  Paper,
  MenuItem,
  Select,
  InputLabel,
  FormControl,
  Box,
  FormHelperText,
  InputAdornment,
  IconButton,
  Input,
} from "@material-ui/core";
import { defaultValue } from "../../model/chart.model";
import ArrowBackIcon from "@material-ui/icons/ArrowBack";
import SaveIcon from "@material-ui/icons/Save";

// TODO - BOTTONE DA FARE COME?
// import { setButton } from "app/shared/layout/floating/floating.reducer";
import { DialogContent, DialogActions, Dialog } from "@material-ui/core";
import Table from "@material-ui/core/Table";
import TableBody from "@material-ui/core/TableBody";
import TableCell from "@material-ui/core/TableCell";
import TableHead from "@material-ui/core/TableHead";
import TableRow from "@material-ui/core/TableRow";
import { isTypeMap } from "../../utils/chart-utils";
import { isEmpty } from "lodash";
import { Clear } from "@material-ui/icons";

export interface IChartUpdateProps
  extends StateProps,
    DispatchProps,
    RouteComponentProps<{ id: string }> {}

export const ChartUpdate = (props /*: IChartUpdateProps*/) => {
  const [isDialogOpen, setDialogOpen] = useState(false);
  const [viewId, setViewId] = useState("");
  const [isNew, setIsNew] = useState(
    !props.match.params || !props.match.params.id
  );
  const { chartEntity, chartViews, loading } = props;
  const [values, setValues] = useState(defaultValue);
  const [mappingValues, setMappingValues] = useState([]);
  const [metadata, setMetadata] = useState("");

  const back = () => {
    props.history.goBack();
  };

  const handleClose = () => {
    props.history.push("/chart");
  };

  useEffect(() => {
    setValues(chartEntity);
    if (!isNew && chartEntity.title) {
      setMappingValues(chartEntity.mapping);
      setViewId(chartEntity.chartView.id);
      setMetadata(chartEntity.mapping.metadata || "");
    }
  }, [chartEntity]);

  // TODO - PERCHE IL RESET DA PROBLEMI??
  useEffect(() => {
    if (isNew) {
      props.reset();
    } else {
      props.getEntity(props.match.params.id);
    }
    // props.setButton([]);
    // props.getChartViews();
    // return () => props.reset();
  }, []);

  useEffect(() => {
    if (props.updateSuccess) {
      handleClose();
    }
  }, [props.updateSuccess]);

  const saveEntity = (e) => {
    mappingValues["metadata"] = isEmpty(metadata.trim())
      ? null
      : metadata.trim();

    e.preventDefault();
    const entity = {
      ...chartEntity,
      ...values,
      mapping: mappingValues,
    };

    if (isNew) {
      props.createEntity(entity);
    } else {
      props.updateEntity(entity);
    }
  };

  const handleChange = (e) =>
    setValues({ ...values, [e.target.name]: e.target.value });

  const handleChangeView = (e) => {
    const chartSelected = props.chartViews.find((i) => i.id === e.target.value);
    setValues({ ...values, chartView: chartSelected });
    const mappedValues = [];
    // TODO
    // chartSelected.dataSetFields.forEach((item) => (mappedValues[item] = ""));
    setMappingValues(mappedValues);
    setViewId(e.target.value);
  };

  return (
    <div>
      {loading ? (
        <p>
          <Translate contentKey="Loading">Loading</Translate>...
        </p>
      ) : (
        <Grid container justify="center">
          <Grid item xs={10} md={6} component={Paper}>
            <Box p={3}>
              <form onSubmit={saveEntity}>
                <Grid
                  container
                  spacing={2}
                  direction="row"
                  justify="center"
                  alignItems="center"
                >
                  <Grid item xs={12}>
                    <TextField
                      margin="normal"
                      label={translate("efraudyApp.chart.title")}
                      required
                      name="title"
                      fullWidth
                      value={values.title || ""}
                      onChange={handleChange}
                    />
                  </Grid>
                  <Grid item xs={12}>
                    <TextField
                      margin="normal"
                      label={translate("efraudyApp.chart.description")}
                      size="small"
                      name="description"
                      fullWidth
                      value={values.description || ""}
                      onChange={handleChange}
                    />
                  </Grid>
                  <Grid item xs={12}>
                    <TextField
                      margin="normal"
                      label={translate("efraudyApp.chart.query")}
                      multiline
                      name="query"
                      fullWidth
                      value={values.query || ""}
                      onChange={handleChange}
                      required
                    />
                    <FormHelperText>
                      <Translate contentKey={"efraudyApp.chart.insertQuery"}>
                        Insert a valid cypher statement
                      </Translate>
                    </FormHelperText>
                  </Grid>
                  <Grid
                    container
                    item
                    xs={12}
                    alignItems="center"
                    justify="center"
                  >
                    <Grid item xs={10}>
                      <FormControl fullWidth>
                        <InputLabel id="chartView">
                          <Translate contentKey="efraudyApp.chart.selectTypeChart">
                            Tipo di grafico
                          </Translate>
                        </InputLabel>
                        <Select
                          name="chartView"
                          labelId="status"
                          onChange={handleChangeView}
                          value={viewId}
                        >
                          {chartViews.map((chartView, i) => (
                            <MenuItem
                              key={`chart-view-${i}`}
                              value={chartView.id}
                            >
                              {chartView.name}
                            </MenuItem>
                          ))}
                        </Select>
                      </FormControl>
                    </Grid>
                    <Grid item xs={2} style={{ textAlign: "center" }}>
                      <Button
                        color="primary"
                        variant="outlined"
                        size="small"
                        className="hand"
                        onClick={() => setDialogOpen(true)}
                      >
                        <Translate contentKey="efraudyApp.chart.info">
                          INFO
                        </Translate>
                      </Button>
                    </Grid>
                    <Dialog
                      maxWidth="md"
                      fullWidth
                      open={isDialogOpen}
                      onClose={() => setDialogOpen(false)}
                    >
                      <DialogContent>
                        <Table>
                          <TableHead>
                            <TableRow>
                              <TableCell align="center">
                                <Translate contentKey="efraudyApp.chartView.name">
                                  Nome
                                </Translate>
                              </TableCell>
                              <TableCell align="center">
                                <Translate contentKey="efraudyApp.chartView.image">
                                  Immagine
                                </Translate>
                              </TableCell>
                              <TableCell align="center">
                                <Translate contentKey="efraudyApp.chartView.documentation">
                                  Documentazione
                                </Translate>
                              </TableCell>
                            </TableRow>
                          </TableHead>
                          <TableBody>
                            {chartViews.map((chartView, i) => (
                              <TableRow key={`chartView-${i}`}>
                                <TableCell align="center">
                                  {chartView.name}
                                </TableCell>
                                <TableCell align="center">
                                  <img
                                    src={`/content/images/chart-view/${chartView.name}.png`}
                                    width="75"
                                    height="75"
                                    alt=""
                                  />
                                </TableCell>
                                <TableCell align="center">
                                  <a
                                    rel="noopener noreferrer"
                                    target="_blank"
                                    href={chartView && chartView.url}
                                  >
                                    {chartView.url}
                                  </a>
                                </TableCell>
                              </TableRow>
                            ))}
                          </TableBody>
                        </Table>
                      </DialogContent>
                      <DialogActions>
                        <Button
                          onClick={() => setDialogOpen(false)}
                          color="primary"
                        >
                          <Translate contentKey="efraudyApp.chartView.close">
                            CHIUDI
                          </Translate>
                        </Button>
                      </DialogActions>
                    </Dialog>
                  </Grid>
                  <Grid item xs={12}>
                    {viewId !== "" &&
                      values.chartView &&
                      values.chartView.dataSetFields &&
                      values.chartView.dataSetFields.map((dataItem, i) => (
                        <Grid item xs={12} key={`dataItem-${dataItem}`}>
                          <TextField
                            label={dataItem}
                            name={`mapping.${dataItem}`}
                            value={mappingValues ? mappingValues[dataItem] : ""}
                            fullWidth
                            placeholder={
                              isTypeMap(values.chartView)
                                ? `${translate(
                                    `efraudyApp.chart.coord${dataItem}`
                                  )}`
                                : `${translate(
                                    "efraudyApp.chart.dataItem"
                                  )} - ${dataItem}`
                            }
                            // todo
                            // onChange={(e) => {
                            //   setMappingValues({
                            //     ...mappingValues,
                            //     [dataItem]: e.target.value,
                            //   });
                            // }}
                            required
                          />
                          {isTypeMap(values.chartView) ? (
                            <FormHelperText>
                              <Translate
                                contentKey={`efraudyApp.chart.insertCoord${dataItem}`}
                              >
                                Inserire coordinata {dataItem}
                              </Translate>
                            </FormHelperText>
                          ) : (
                            <FormHelperText>
                              <Translate
                                contentKey={"efraudyApp.chart.insertDataItem"}
                              >
                                Inserire il data item:
                              </Translate>{" "}
                              {dataItem}
                            </FormHelperText>
                          )}
                        </Grid>
                      ))}
                  </Grid>
                  <Grid item xs={12}>
                    <FormControl fullWidth>
                      <InputLabel>
                        <Translate contentKey={"efraudyApp.chart.metadata"}>
                          efraudyApp.chart.metadata
                        </Translate>
                      </InputLabel>
                      <Input
                        name={`metadata`}
                        value={metadata || ""}
                        placeholder={translate(`efraudyApp.chart.metadata`)}
                        onChange={(e) => setMetadata(e.target.value)}
                        endAdornment={
                          <InputAdornment position="end">
                            <IconButton
                              aria-label="clear value"
                              onClick={(e) => setMetadata("")}
                            >
                              <Clear />
                            </IconButton>
                          </InputAdornment>
                        }
                      />
                    </FormControl>
                    <FormHelperText>
                      <Translate contentKey={"efraudyApp.chart.insertMetadata"}>
                        Inserire il campo per i metadata
                      </Translate>
                    </FormHelperText>
                  </Grid>
                  <Grid container component={Box} mt={4} spacing={2}>
                    <Grid item xs={3}>
                      <Button
                        onClick={back}
                        variant="contained"
                        size="large"
                        fullWidth
                        startIcon={<ArrowBackIcon />}
                        color="secondary"
                      >
                        <Translate contentKey="entity.action.back">
                          Back
                        </Translate>
                      </Button>
                    </Grid>
                    <Grid item xs={3}>
                      <Button
                        type="submit"
                        variant="contained"
                        size="large"
                        fullWidth
                        startIcon={<SaveIcon />}
                        color="primary"
                      >
                        <Translate contentKey="entity.action.save">
                          Save
                        </Translate>
                      </Button>
                    </Grid>
                  </Grid>
                </Grid>
              </form>
            </Box>
          </Grid>
        </Grid>
      )}
    </div>
  );
};

// TODO - TIPIZZARE
const mapStateToProps = (storeState) => ({
  chartViews: storeState.chartView.entities,
  chartEntity: storeState.chart.entity,
  loading: storeState.chart.loading,
  updating: storeState.chart.updating,
  updateSuccess: storeState.chart.updateSuccess,
});

const mapDispatchToProps = {
  // getChartViews,
  getEntity,
  updateEntity,
  createEntity,
  reset,
  // setButton,
};

type StateProps = ReturnType<typeof mapStateToProps>;
type DispatchProps = typeof mapDispatchToProps;

export default connect(mapStateToProps, mapDispatchToProps)(ChartUpdate);
