import {
  ChartViewType,
  DEFAULT_CHART_SIZE,
  REST_HEIGHT,
  HEIGHT_LEGEND,
} from "../utils/constants";

// TODO - NECESSARIO?
import chroma from "chroma-js";

import * as vis from "react-vis";
import { Hint } from "react-vis";
import {
  BoxWithEvent,
  VisWithEvent,
  VisWithPlotAndEvent,
} from "./components-utils";
import { isEmpty } from "lodash";

export const graphWithLabel = (chartView) =>
  chartView.dataSetFields && chartView.dataSetFields.includes("label");

export const createChartFromEntity = (
  chartEntity,
  chartDataset,
  size = null
) => {
  const makeChart = [];
  const sizeChart =
    size && size[1] ? size[1] - REST_HEIGHT : DEFAULT_CHART_SIZE;
  const innerSizeChart =
    sizeChart - (graphWithLabel(chartEntity) ? HEIGHT_LEGEND : 0);
  const hintOpenTag = `chartValues.id === id && <Hint padAngle={0.04} className="hint" value={chartValues.hint}>`;

  const metadatahints =
    !isEmpty(chartDataset) && chartDataset[0].metadata
      ? Object.keys(chartDataset[0].metadata)
          .map(
            (singleMetadata) =>
              `<p>${singleMetadata}: {chartValues.hint.metadata.${singleMetadata} } </p>`
          )
          .join("")
      : "";

  if (chartEntity.fields) {
    makeChart.push(chartEntity.fields.map((item) => `<vis.${item}/>`).join(""));
  }
  const dataField = `vis.${chartEntity.dataField} data={chartDataset} colorType="literal" className="innerChart" height={${innerSizeChart}} width={${innerSizeChart}}`;

  if (chartEntity.chartViewType === ChartViewType.XYPLOT) {
    makeChart.unshift(
      `<VisWithPlotAndEvent xType="${
        chartEntity.xType || "ordinal"
      }" height={${sizeChart}} eventOut={eventOut} eventOver={eventOver}
        dataFieldComponent={<${dataField}/>}
          fieldComponents={ <vis.FlexibleXYPlot>`
    );

    makeChart.push(`{${hintOpenTag}<p>x: {chartValues.hint.x}</p><p>y: {chartValues.hint.y}</p>${metadatahints}</Hint>}
      </vis.FlexibleXYPlot> } />`);
  } else {
    makeChart.push(
      `<VisWithEvent eventOver={eventOver} eventOut={eventOut} component={<${dataField}>
        {${hintOpenTag}<p>{chartValues.percentage} %</p><p>{chartValues.hint.label}</p>${metadatahints}</Hint>}
      </vis.${chartEntity.dataField}> } />`
    );
    makeChart.unshift(`<BoxWithEvent style={{height: ${innerSizeChart}}}>`);
    makeChart.push(`</BoxWithEvent>`);
  }

  if (graphWithLabel(chartEntity)) {
    makeChart.push(
      `<BoxWithEvent eventEnter={disableDragEvent} eventLeave={enableDragEvent}><vis.DiscreteColorLegend orientation="horizontal" height={${HEIGHT_LEGEND}} items = {colorAndItems.items} /></BoxWithEvent>`
    );
  }
  return makeChart.join("");
};

export const isTypeMap = (chartView) =>
  chartView &&
  chartView.chartViewType &&
  chartView.chartViewType.includes("MAP");

export const defaultColorAndItems = () => {
  return { items: [] };
};

export const defaultChartValues = () => {
  return {
    hint: { angle: null, x: null, y: null },
    percentage: null,
    id: null,
  };
};

export const calculatePercentage = (chartDataset, v) => {
  let tot = 0,
    item = 0;
  chartDataset.forEach((i) => {
    tot = tot + i.angle;
    if (i.label === v.label) {
      item = i.angle;
    }
  });
  return Math.round((item / tot) * 100);
};

export const parserComponents = {
  VisWithEvent,
  VisWithPlotAndEvent,
  vis,
  BoxWithEvent,
  Hint,
};

export const isHintChanged = (previousHint, currentValueHint) =>
  previousHint === defaultChartValues().hint ||
  Object.entries(currentValueHint).some(
    (i) => previousHint[i[0]] !== currentValueHint[i[0]]
  );

export const getConfByChartId = (id) => {
  const confChart = sessionStorage.getItem("confChart");
  return (confChart && JSON.parse(confChart)[id]) || defaultColorAndItems();
};

export const saveChartConfig = (id, chartDataset) => {
  const colors = JSON.parse(sessionStorage.getItem("confChart")) || {};
  const colorAndItems = defaultColorAndItems();
  chartDataset.forEach((element) => {
    const itemConf = getConfByChartId(id).items;
    const color = (
      itemConf.find((item) => item.title === element.label) || {
        color: chroma.random().hex(),
      }
    ).color;
    colorAndItems.items.push({ title: element.label, color, strokeWidth: 6 });
    element["color"] = color;
  });
  colors[id] = colorAndItems;
  sessionStorage.setItem("confChart", JSON.stringify(colors));
};
